package meerkat.voting.gui.configuration;

import com.google.protobuf.ByteString;
import meerkat.protobuf.BallotQuestionUIElementOuterClass;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;



/**
 * Created by Vladimir Eliezer Tokarev on 10/4/2016.
 * This class creates QuestionBallotUIElement from given configuration file
 */
public class VotingBoothConfigurationCreator {
    private int CURRENT_NUMBER_OF_QUESTIONS_REPRESENTED_TO_VOTER = 2 ;
    public static String WRONG_QUESTIONS_AMOUNT_ERROR_MESSAGE = "The configuration file had too many ballot question objects.";
    public static String UNSUPPORTED_VALUE_TYPE_ERROR_MESSAGE = "The given question type doesnt supported in current version";
    private String IMAGES_TYPE = "png";

    /**
     * Creates VotingBoothConfiguarion object form file that given as paramter
     * @param configPath the path to the json configuration
     * @return configuraiton object
     */
    public VotingBoothConfiguration CreateConfiguration(String configPath) throws IOException, JSONException {
        VotingBoothConfiguration config = new VotingBoothConfiguration();
        JSONArray array = this.getJsonFromPath(configPath);

        if (array.length() != CURRENT_NUMBER_OF_QUESTIONS_REPRESENTED_TO_VOTER) {
            throw new IOException(WRONG_QUESTIONS_AMOUNT_ERROR_MESSAGE);
        } else {
            try {
                BallotQuestionUIElementOuterClass.BallotQuestionUIElement ballotQuestionUIElement = this.convertJSONObjToBallotQuestion(array.getJSONObject(0));
                config.NameSelectionQuestion = ballotQuestionUIElement;

                ballotQuestionUIElement = this.convertJSONObjToBallotQuestion(array.getJSONObject(1));
                config.NameSelectionByPictureQuestion = ballotQuestionUIElement;
            }
            catch (JSONException e) {
                e.printStackTrace();
            }
        }

        return config;
    }

    /**
     * Gets array of object that contains the data of the BallotAQuestions
     * @param path the path to the configuration file
     * @return JSONArray which represnets array of ballot Questions
     * @throws FileNotFoundException will be thrown in the config file not exists
     * @throws JSONException will be thrown if the parsing will fail
     */
    private JSONArray getJsonFromPath(String path) throws FileNotFoundException, JSONException {
        String content = new Scanner(new File(path)).useDelimiter("\\Z").next();
        JSONObject jsonObject = new JSONObject(content);
        return (JSONArray)jsonObject.get("questions");
    }

    /**
     * converts the given JSONObject into BallotQuestionUIElement
     * @param object the object to convert
     * @return BallotQuestion parsed fron JSONObject
     */
    private BallotQuestionUIElementOuterClass.BallotQuestionUIElement convertJSONObjToBallotQuestion(JSONObject object) throws JSONException, IOException {
        boolean randomize = object.getBoolean("randomize");

        // Question part
        JSONObject question = object.getJSONObject("question");
        int questionType = question.getInt("type");
        ByteString questionValue = this.convertToByteStringBasedOnType(questionType, question.getString("questionValue"));

        // Answers Part
        JSONObject answers = object.getJSONObject("answers");
        int answersType = answers.getInt("type");
        JSONArray answersValues = answers.getJSONArray("answersValue");
        List<ByteString> byteStrings = new ArrayList<>();
        for (int i  = 0 ; i < answersValues.length(); i ++){
            byteStrings.add(this.convertToByteStringBasedOnType(answersType, answersValues.getString(i)));
        }

        // Create the UIQuestion and set the values parsed from jason
        BallotQuestionUIElementOuterClass.UIQuestion uiQuestion = BallotQuestionUIElementOuterClass.UIQuestion.newBuilder()
                .setQuestionType(questionType)
                .setQuestion(BallotQuestionUIElementOuterClass.Question.newBuilder()
                        .setQuestion(questionValue))
                .build();

        // Create UIAnswers and set the values parsed from jason
        BallotQuestionUIElementOuterClass.UIAnswers.Builder uiAnswersBuilder = BallotQuestionUIElementOuterClass.UIAnswers.newBuilder()
                .setAnswersType(answersType);
        BallotQuestionUIElementOuterClass.ListOfAnswers.Builder listOfAnswers = BallotQuestionUIElementOuterClass.ListOfAnswers.newBuilder();
        for(ByteString byteString : byteStrings){
            listOfAnswers.addAnswers(byteString);
        }
        uiAnswersBuilder.setAnswers(listOfAnswers);

        return BallotQuestionUIElementOuterClass.BallotQuestionUIElement.newBuilder()
                .setQuestion(uiQuestion)
                .setAnswers(uiAnswersBuilder.build())
                .setRandomizeListOrder(randomize)
                .build();
    }

    /**
     * Tries to get convert the given question to ByteString else throws excetion
     * @param questionType the type of the question (can be 0 - text, 1 - audio, 2 - image)
     * @param questionValue the values itseld (in case the type is 1 or 2 the question value will be a path to the file)
     * @return ByteString which is or the value or the file in given path
     * @throws IOException will be thrown if the given type is unsupported
     */
    private ByteString convertToByteStringBasedOnType(int questionType, String questionValue) throws IOException {
        if (questionType == BallotQuestionUIElementOuterClass.ValueType.TEXT_TYPE.getNumber()) {
            return ByteString.copyFromUtf8(questionValue);
        }
        if (questionType == BallotQuestionUIElementOuterClass.ValueType.IMAGE_TYPE.getNumber()) {
            return ByteString.copyFrom(getFileBytes(questionValue));
        }
        else {
            throw new IOException(UNSUPPORTED_VALUE_TYPE_ERROR_MESSAGE);
        }
    }

    /**
     * Converts the image from given path to bytes array
     * @param filepath the path to the wanted picture
     * @return byte array of the image
     * @throws IOException will be thrown if the given file doesnt exist
     */
    private byte[] getFileBytes(String filepath) throws IOException {
        byte[] imageInByte;
        BufferedImage originalImage = ImageIO.read(new File(filepath));

        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        ImageIO.write(originalImage, IMAGES_TYPE, baos);
        baos.flush();
        imageInByte = baos.toByteArray();
        baos.close();
        return imageInByte;
    }
}


